// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/WFinder.hh"
#include "Rivet/Projections/ZFinder.hh"
#include "Rivet/Projections/FastJets.hh"
#include "Rivet/Projections/VetoedFinalState.hh"
// BOOST
#include <boost/lexical_cast.hpp>


namespace Rivet {

  struct Plots {
    string ref;
    Histo1DPtr comp[2]; // (de)nominator components
    Scatter2DPtr ratio; // Rjets plot
  };

  typedef map<string, Plots>::iterator it_type;

  class ATLAS_2014_I1312627 : public Analysis {

  public:

    /// Constructor
    ATLAS_2014_I1312627(std::string name="ATLAS_2014_I1312627")
      : Analysis(name)
    {
      _mode = 0; // using electron channel for combined data by default
      setNeedsCrossSection(true);
    }


  public:

    /// @name Analysis methods
    //@{
    
    /// Book histograms and initialise projections before the run
    void init() {
      FinalState fs;
      
      Cut cuts;
      if(_mode == 2) { // muon channel
        cuts = (Cuts::pT > 25.0*GeV) & Cuts::etaIn(-2.4, 2.4);
      } else if(_mode) { // electron channel
        cuts = (Cuts::pT > 25.0*GeV) & ( Cuts::etaIn(-2.47, -1.52) | Cuts::etaIn(-1.37, 1.37) | Cuts::etaIn(1.52, 2.47) );
      } else { // combined data extrapolated to common phase space
        cuts = (Cuts::pT > 25.0*GeV) & Cuts::etaIn(-2.5, 2.5);
      }

      // bosons
      WFinder wfinder(fs, cuts, _mode > 1? PID::MUON : PID::ELECTRON, 40.0*GeV, MAXDOUBLE, 0.0*GeV, 0.1,
                      WFinder::CLUSTERNODECAY, WFinder::NOTRACK, WFinder::TRANSMASS);
      addProjection(wfinder, "WF");

      ZFinder zfinder(fs, cuts, _mode > 1? PID::MUON : PID::ELECTRON, 66.0*GeV, 116.0*GeV, 0.1, ZFinder::CLUSTERNODECAY, ZFinder::NOTRACK);
      addProjection(zfinder, "ZF");

      // jets
      VetoedFinalState jet_fs(fs);
      jet_fs.addVetoOnThisFinalState(getProjection<WFinder>("WF"));
      jet_fs.addVetoOnThisFinalState(getProjection<ZFinder>("ZF"));
      FastJets jets(jet_fs, FastJets::ANTIKT, 0.4);
      jets.useInvisibles(true);
      addProjection(jets, "Jets");

      _suff = string("-y0") + boost::lexical_cast<string>(_mode + 1);

      // book Rjets plots
      hInit("Njets_incl",  "d01"); // inclusive number of jets
      hInit("Njets_excl",  "d04"); // exclusive number of jets
      hInit("Pt1_N1incl",  "d05"); // leading jet pT, at least 1 jet
      hInit("Pt1_N1excl",  "d06"); // leading jet pT, exactly 1 jet
      hInit("Pt1_N2incl",  "d07"); // leading jet pT, at least 2 jets
      hInit("Pt1_N3incl",  "d08"); // leading jet pT, at least 3 jets
      hInit("Pt2_N2incl",  "d09"); // subleading jet pT, at least 2 jets
      hInit("Pt3_N3incl",  "d10"); // sub-subleading jet pT, at least 3 jets
      hInit("ST_N2incl",   "d11"); // scalar jet pT sum, at least 2 jets
      hInit("ST_N2excl",   "d12"); // scalar jet pT sum, exactly 2 jets
      hInit("ST_N3incl",   "d13"); // scalar jet pT sum, at least 3 jets
      hInit("ST_N3excl",   "d14"); // scalar jet pT sum, exactly 3 jets
      hInit("DR_N2incl",   "d15"); // deltaR(j1, j2), at least 2 jets
      hInit("DPhi_N2incl", "d16"); // deltaPhi(j1, j2), at least 2 jets
      hInit("Mjj_N2incl",  "d17"); // mjj, at least 2 jets
      hInit("Rap1_N1incl", "d18"); // leading jet rapidity, at least 1 jet
      hInit("Rap2_N2incl", "d19"); // subleading jet rapidity, at least 2 jets
      hInit("Rap3_N3incl", "d20"); // sub-subleading jet rapidity, at least 3 jets

      // also book nominator and denominator for Rjets plots
      for(it_type it = _h.begin(); it != _h.end(); ++it) {
        it->second.comp[0] = bookHisto1D( it->second.ref + "2" + _suff, *(it->second.ratio) );
        it->second.comp[1] = bookHisto1D( it->second.ref + "3" + _suff, *(it->second.ratio) );
      }
    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {

      const double weight = event.weight();

      // retrieve boson candidate
      const WFinder& wf = applyProjection<WFinder>(event, "WF");
      const ZFinder& zf = applyProjection<ZFinder>(event, "ZF");
      if(wf.empty() && zf.empty())  vetoEvent; 

      // retrieve jets
      const JetAlg& jetfs = applyProjection<JetAlg>(event, "Jets");
      Jets all_jets = jetfs.jetsByPt(30.0*GeV, MAXDOUBLE, -4.4, 4.4, RAPIDITY);

      if(!zf.empty()) {
        const Particles& leptons = zf.constituents();
        if( leptons.size() == 2 &&
            leptons[0].charge() * leptons[1].charge() < 0. &&
            deltaR(leptons[0], leptons[1]) > 0.2 )  fillPlots(leptons, all_jets, weight, 1);
      }
      if(!wf.empty()) {
        const Particles& leptons = wf.constituentLeptons();
        if( leptons.size() == 1 &&
            wf.constituentNeutrino().pT() > 25.0*GeV &&
            wf.mT() > 40.0*GeV )  fillPlots(leptons, all_jets, weight, 0);
      }
    }

    void fillPlots(const Particles& leptons, Jets& all_jets, const double &weight, int isZ) {
      // do jet-lepton overlap removal
      Jets jets;
      foreach(Jet &j, all_jets) {
        bool keep = true;
        foreach(const Particle& l, leptons)  keep &= deltaR(j, l) > 0.5;
        if(keep)  jets.push_back(j);
      }

      int njets = jets.size();
      
      double ST = 0.0; // scalar pT sum of all selected jets
      for(int i = 0; i < njets; ++i)  ST += jets[i].pT() * GeV;

      _h["Njets_excl"].comp[isZ]->fill(njets + 0.5, weight);
      for(int i = 0; i <= njets; ++i) {
        _h["Njets_incl"].comp[isZ]->fill(i + 0.5, weight);
      }

      if(njets) {
        double pT1  = jets[0].pT() * GeV;
        double rap1 = jets[0].absrap();
        _h["Pt1_N1incl" ].comp[isZ]->fill(pT1,  weight);
        _h["Rap1_N1incl"].comp[isZ]->fill(rap1, weight);
        if(njets == 1)  _h["Pt1_N1excl"].comp[isZ]->fill(pT1, weight);

        if(njets > 1) {
          double pT2  = jets[1].pT() * GeV;
          double rap2 = jets[1].absrap();
          double dR   = deltaR(jets[0], jets[1]);
          double dPhi = deltaPhi(jets[0], jets[1]);
          double mjj  = (jets[0].momentum() + jets[1].momentum()).mass() * GeV;
          _h["Pt1_N2incl" ].comp[isZ]->fill(pT1,  weight);
          _h["Pt2_N2incl" ].comp[isZ]->fill(pT2,  weight);
          _h["Rap2_N2incl"].comp[isZ]->fill(rap2, weight);
          _h["DR_N2incl"  ].comp[isZ]->fill(dR,   weight);
          _h["DPhi_N2incl"].comp[isZ]->fill(dPhi, weight);
          _h["Mjj_N2incl" ].comp[isZ]->fill(mjj,  weight);
          _h["ST_N2incl"  ].comp[isZ]->fill(ST,   weight);
          if(njets == 2)  _h["ST_N2excl"].comp[isZ]->fill(ST, weight);
        
          if(njets > 2) {
            double pT3  = jets[2].pT() * GeV;
            double rap3 = jets[2].absrap();
            _h["Pt1_N3incl" ].comp[isZ]->fill(pT1,  weight);
            _h["Pt3_N3incl" ].comp[isZ]->fill(pT3,  weight);
            _h["Rap3_N3incl"].comp[isZ]->fill(rap3, weight);
            _h["ST_N3incl"  ].comp[isZ]->fill(ST,   weight);
            if(njets == 3)  _h["ST_N3excl"].comp[isZ]->fill(ST, weight);
          }
        }
      }
    }

    void hInit(string label, string ident) {
      string pre = ident + "-x0";
      _h[label].ref = pre;
      _h[label].ratio = bookScatter2D(pre + "1" + _suff, true);
    }

    /// Normalise histograms etc., after the run
    void finalize() {
      ///  Normalise, scale and otherwise manipulate histograms here
      const double sf( crossSection() / sumOfWeights() );
      for(it_type it = _h.begin(); it != _h.end(); ++it) {
        scale(it->second.comp[0], sf);
        scale(it->second.comp[1], sf);
        divide(it->second.comp[0], it->second.comp[1], it->second.ratio);
      }
    }
    //@}

  protected:
	  
    // Data members like post-cuts event weight counters go here
    size_t _mode;
    string _suff;

  private:

    /// @name Histograms
    //@{
    map<string, Plots> _h;
    //@}

  };
  

  class ATLAS_2014_I1312627_EL : public ATLAS_2014_I1312627 {
  public:
    ATLAS_2014_I1312627_EL()
      : ATLAS_2014_I1312627("ATLAS_2014_I1312627_EL")
    {
      _mode = 1;
    }
  };

  class ATLAS_2014_I1312627_MU : public ATLAS_2014_I1312627 {
  public:
    ATLAS_2014_I1312627_MU()
      : ATLAS_2014_I1312627("ATLAS_2014_I1312627_MU")
    {
      _mode = 2;
    }
  };

  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2014_I1312627);
  DECLARE_RIVET_PLUGIN(ATLAS_2014_I1312627_EL);
  DECLARE_RIVET_PLUGIN(ATLAS_2014_I1312627_MU);

} 
